unit BrowserWidget;
{$codepage UTF8}

interface

uses
    AnsiParser,
    DrawUtils,
    Logger,
    SideBarWidget,
    TurboGopherApplication,

    Classes,
    Objects,
    StrUtils,
    SysUtils,
    UDrivers,
    UViews,
    video;
type

    TBrowserString = array of TEnhancedVideoCell;

    TBrowserWidget = object(TScroller)
        constructor Init(
            var TheApp: TTurboGopherApplication;
            SideBarWidget: PSideBarWidget;
            Bounds: TRect;
            AHScrollBar, AVScrollBar: PScrollBar
        );
        procedure Add(text: UnicodeString);
        procedure Draw; virtual;
        procedure Reset;
        private
            AnsiParser: TAnsiParser;
            App: TTurboGopherApplication;
            Lines: array of TBrowserString;
            CurrentForegroundColor: byte;
            CurrentBackgroundColor: byte;
            DefaultCell: TEnhancedVideoCell;
            SideBar: PSideBarWidget;
            const defaultFgColor = 2;
            const defaultBgColor = 0;

    end;
    PBrowserWidget = ^TBrowserWidget;

implementation

{ TBrowserWidget }

constructor TBrowserWidget.Init(
            var TheApp: TTurboGopherApplication;
            SideBarWidget: PSideBarWidget;
            Bounds: TRect;
            AHScrollBar, AVScrollBar: PScrollBar
        );
begin
    TScroller.Init(Bounds, AHScrollBar, AVScrollBar);
    App := TheApp;
    SideBar := SideBarWidget;
    CurrentForegroundColor := defaultFgColor;
    CurrentBackgroundColor := defaultBgColor;
    GrowMode := gfGrowHiX + gfGrowHiY;
    SetLimit(128, 10);
    DefaultCell := default(TEnhancedVideoCell);
    DefaultCell.ForegroundColor := DefaultFgColor;
    DefaultCell.BackgroundColor := DefaultBgColor;
    DefaultCell.EGC_SingleChar := ' ';
    DefaultCell.Attribute := 0;
    DefaultCell.EnhancedVideoAttributes := [];
    AnsiParser.Init(App, DefaultCell);
end;

procedure TBrowserWidget.Add(text: UnicodeString);
var
  CC, PC: UnicodeChar;
  NewChar: TEnhancedVideoCell;
  I: SizeInt;
  Line: TBrowserString;
  Logger: PLogger;

begin
    Logger := App.GetLogger();
    PC := chr(0);
    NewChar := default(TEnhancedVideoCell);
    Line := default(TBrowserString);
    { determine attributes for characters }
    NewChar.ForegroundColor := CurrentForegroundColor;
    NewChar.BackgroundColor := CurrentBackgroundColor;
    { iterate characters in text to be added }
    for I := 0 to (Length(text) - 1) do
    begin
        (* If the character is a new line, allocate a new line and save the
           previous one. *)
        CC := text[I + 1];

        if (ord(CC) = 10) or (ord(CC) = 13) then
        begin
            (* If the previous character was a <CR> and the current character is
               an <LF> then ignore and don't add an additional new line. *)
            if not ((ord(PC) = 13) and (ord(CC) = 10)) then
            begin
                SetLength(Lines, Length(Lines) + 1);
                Lines[Length(Lines) - 1] := Line;
                Line := default(TBrowserString);
            end;
            PC := CC;
            continue;
        end;

        (* Parse CSI escapes. *)
        if AnsiParser.ParseChar(CC, NewChar, text, I) <> True then
        begin
            PC := CC;
            continue;
        end;
        NewChar.EGC_SingleChar := text[I + 1];
        SetLength(Line, Length(Line) + 1);
        Line[Length(Line) - 1] := NewChar;
        PC := CC;
    end;
    { add the line to our lines array. }
    SetLength(Lines, Length(Lines) + 1);
    Lines[Length(Lines) - 1] := Line;
end;

procedure TBrowserWidget.Draw;
var
  I: Integer;
  DrawBuffer: TDrawBuffer;
  DrawBufferIndex, X, Y, LineCount, LongestLine: SizeInt;
  Rect: TRect;
  OChar: TEnhancedVideoCell;
begin
    Rect := default(TRect);
    GetBounds(Rect);
    SideBar^.SetDelta(Delta.Y);
    SideBar^.Draw;
    DrawBuffer := default(TDrawBuffer);
    DrawBufferIndex := 0;
    LongestLine := 0;
    LineCount := Length(Lines);
    { clear the screen }
    OChar := DefaultCell;
    AddToDrawBuf(DrawBuffer, OChar, Size.X);
    WriteLine(0, 0, Size.X, Size.Y, DrawBuffer);
    DrawBuffer := default(TDrawBuffer);
    { render characters }
    for Y := 0 to Rect.B.Y do
    begin
        I := Delta.Y + Y;
        if (I < LineCount) and (Lines[I] <> nil) then
        begin
            if Length(Lines[I]) > LongestLine then LongestLine := Length(Lines[I]);
            for X := Delta.X to (Delta.X + Size.X) do
            begin
                if (Length(Lines[I]) - 1 < X) then continue;
                OChar := Lines[I][X];
                SetInDrawBuf(DrawBuffer, OChar, DrawBufferIndex);
                DrawBufferIndex += 1;
            end;
            WriteBuf(0, Y, DrawBufferIndex, 1, DrawBuffer);
            DrawBufferIndex := 0;
        end;
    end;
    SetLimit(LongestLine, LineCount);
end;

procedure TBrowserWidget.Reset;
begin
    SetLength(Lines, 0);
    AnsiParser.Reset;
end;

end.

