unit SideBarWidget;
{$codepage UTF8}

interface

uses

    DrawUtils,
    GopherClient,
    TurboGopherApplication,

    Classes,
    Objects,
    SysUtils,
    UDrivers,
    UViews,
    video;

type

    PSideBarWidget = ^TSideBarWidget;
    TSideBarWidget = object(TView)
        constructor Init(
            var TheApp: TTurboGopherApplication;
            Bounds: TRect
        );
        procedure Add(Item: TGopherMenuItem);
        procedure Draw; virtual;
        function GetDelta: SizeInt;
        function GetFirst: SizeInt;
        function GetLast: SizeInt;
        function GetSelected: PGopherMenuItem;
        procedure Reset;
        procedure SetDelta(d: SizeInt);
        procedure SetItems(MenuItems: TGopherMenuItems);
        procedure SelectFirst(var Browser: TScroller);
        procedure SelectNext(var Browser: TScroller);
        procedure SelectPrevious(var Browser: TScroller);
        procedure SelectLast(var Browser: TScroller);
        private
            App: TTurboGopherApplication;
            Delta: SizeInt;
            Items: TGopherMenuItems;
            Rect: TRect;
            SelectedDelta: SizeInt;
            SelectedIndex: SizeInt;
            const
                defaultBackgroundBg = 0;
                defaultBackgroundFg = 15;
                defaultTextBg = 0;
                defaultTextFg = 15;
                defaultDirBg = 0;
                defaultDirFg = 2;
                defaultCsoNsBg = 0;
                defaultCsoNsFg = 15;
                defaultHqxFilerBg = 0;
                defaultHqxFilerFg = 15;
                defaultBinaryBg = 0;
                defaultBinaryFg = 15;
                defaultUuFileBg = 0;
                defaultUuFileFg = 15;
                defaultSearchBg = 0;
                defaultSearchFg = 3;
                defaultTelnetBg = 0;
                defaultTelnetFg = 15;
                defaultCalBg = 0;
                defaultCalFg = 15;
                defaultEventBg = 0;
                defaultEventFg = 15;
                defaultGifBg = 0;
                defaultGifFg = 15;
                defaultHtmlBg = 0;
                defaultHtmlFg = 15;
                defaultSoundBg = 0;
                defaultSoundFg = 15;
                defaultImageBg = 0;
                defaultImageFg = 15;
                defaultMimeBg = 0;
                defaultMimeFg = 15;
                defaultTnBg = 0;
                defaultTnFg = 15;
    end;

implementation

uses BrowserWidget;

{ Helper functions }

function IsSelectable(Item: TGopherMenuItem): Boolean;
begin
    Result := False;
    if Item.ItemType in [
        '0', '1', '2', '4', '5', '6', '7', '8', '9',
        'c', 'e', 'g', 'h', 's', 'I', 'M', 'T'
    ] then Result := True;
end;

{ TSideBarWidget }

constructor TSideBarWidget.Init(
    var TheApp: TTurboGopherApplication;
    Bounds: TRect
);
begin
    Delta := 0;
    SelectedDelta := 0;
    Rect := Bounds;
    TView.Init(Bounds);
    App := TheApp;
    SelectedIndex := 0;
end;

procedure TSideBarWidget.Add(Item: TGopherMenuItem);
begin
    SetLength(Items, Length(Items) + 1);
    Items[Length(Items) - 1] := Item;
end;

procedure TSideBarWidget.Draw;
var
    DrawBuffer: TDrawBuffer;
    I, L: SizeInt;
    Entry: UnicodeString = '';
    Selectable: Boolean = False;
    OChar: TEnhancedVideoCell;
begin
    OChar := default(TEnhancedVideoCell);
    DrawBuffer := default(TDrawBuffer);
    { clear the screen }
    OChar.EGC_SingleChar := chr(32);
    OChar.BackgroundColor := defaultBackgroundBg;
    OChar.ForegroundColor := defaultBackgroundFg;
    AddToDrawBuf(DrawBuffer, OChar, Size.X);
    WriteLine(0, 0, Size.X, Size.Y, DrawBuffer);
    DrawBuffer := default(TDrawBuffer);
    { render characters }
    GetBounds(Rect);
    for L := 0 to Rect.B.Y do
    begin
        Entry := '   ';
        I := Delta + L;
        Selectable := False;
        if I < Length(Items) then
        begin
            Entry := Items[I].ItemType;
            Selectable := IsSelectable(Items[I]);
        end;
        case Entry of
            '0':
            begin
                Entry := ' text  ';
                OChar.ForegroundColor := defaultTextFg;
                OChar.BackgroundColor := defaultTextBg;
            end;
            '1':
            begin
                Entry := ' dir   ';
                OChar.ForegroundColor := defaultDirFg;
                OChar.BackgroundColor := defaultDirBg;
            end;
            '2':
            begin
                Entry := ' cso ns';
                OChar.ForegroundColor := defaultCsoNsFg;
                OChar.BackgroundColor := defaultCsoNsBg;
            end;
            '3':
            begin
                Entry := '       ';
                OChar.ForegroundColor := defaultBackgroundFg;
                OChar.BackgroundColor := defaultBackgroundBg;
            end;
            '4':
            begin
                Entry := ' hqxflr';
                OChar.ForegroundColor := defaultHqxFilerFg;
                OChar.BackgroundColor := defaultHqxFilerBg;
            end;
            '5':
            begin
                Entry := ' binary';
                OChar.ForegroundColor := defaultBinaryFg;
                OChar.BackgroundColor := defaultBinaryBg;
            end;
            '6':
            begin
                Entry := ' uufile';
                OChar.ForegroundColor := defaultUuFileFg;
                OChar.BackgroundColor := defaultUuFileBg;
            end;
            '7':
            begin
                Entry := ' search';
                OChar.ForegroundColor := defaultSearchFg;
                OChar.BackgroundColor := defaultSearchBg;
            end;
            '8':
            begin
                Entry := ' telnet';
                OChar.ForegroundColor := defaultTelnetFg;
                OChar.BackgroundColor := defaultTelnetBg;
            end;
            '9':
            begin
                Entry := ' binary';
                OChar.ForegroundColor := defaultBinaryFg;
                OChar.BackgroundColor := defaultBinaryBg;
            end;
            'c':
            begin
                Entry := ' cal   ';
                OChar.ForegroundColor := defaultCalFg;
                OChar.BackgroundColor := defaultCalBg;
            end;
            'e':
            begin
                Entry := ' event ';
                OChar.ForegroundColor := defaultEventFg;
                OChar.BackgroundColor := defaultEventBg;
            end;
            'g':
            begin
                Entry := ' gifimg';
                OChar.ForegroundColor := defaultGifFg;
                OChar.BackgroundColor := defaultGifBg;
            end;
            'h':
            begin
                Entry := ' html  ';
                OChar.ForegroundColor := defaultHtmlFg;
                OChar.BackgroundColor := defaultHtmlBg;
            end;
            'i':
            begin
                Entry := '       ';
                OChar.ForegroundColor := defaultBackgroundFg;
                OChar.BackgroundColor := defaultBackgroundBg;
            end;
            's':
                begin
                    Entry := ' sound ';
                    OChar.ForegroundColor := defaultSoundFg;
                    OChar.BackgroundColor := defaultSoundBg;
                end;
            'I':
            begin
                Entry := ' image ';
                OChar.ForegroundColor := defaultImageFg;
                OChar.BackgroundColor := defaultImageBg;
            end;
            'M':
            begin
                Entry := ' mime  ';
                OChar.ForegroundColor := defaultMimeFg;
                OChar.BackgroundColor := defaultMimeBg;
            end;
            'T':
            begin
                Entry := ' TN3270';
                OChar.ForegroundColor := defaultTnFg;
                OChar.BackgroundColor := defaultTnBg;
            end
        else
            Entry := '       '
        end;
        if (Selectable = True) and (I = SelectedIndex) then
        begin
            SelectedDelta := I;
            InvertColor(OChar);
        end;
        MoveStr(DrawBuffer, Entry, OChar.Attribute);
        WriteBuf(0, L, Length(Entry), 1, DrawBuffer);
        OChar.ForegroundColor := defaultBackgroundFg;
        OChar.BackgroundColor := defaultBackgroundBg;
        MoveStr(DrawBuffer, '▕', Ochar.Attribute);
        WriteBuf(Length(Entry), L, 1, 1, DrawBuffer);
    end;
end;

function TSideBarWidget.GetDelta: SizeInt;
begin
    Result := SelectedDelta;
end;

function TSideBarWidget.GetFirst: SizeInt;
begin
    Result := 0;
    if Length(Items) = 0 then
    begin
        Exit;
    end;
    for Result := 0 to Length(Items) - 1 do
    begin
        if IsSelectable(Items[Result]) = True then break;
    end;
end;

function TSideBarWidget.GetLast: SizeInt;
begin
    Result := Length(Items) - 1;
    if Result = 0 then
    begin
        Exit;
    end;
    while Result > 0 do
    begin
        if IsSelectable(Items[Result]) = True then break;
        Result := Result - 1;
    end;
end;

function TSideBarWidget.GetSelected: PGopherMenuItem;
begin
    Result := nil;
    if Length(Items) = 0 then Exit;
    if SelectedIndex > Length(Items) - 1 then
    begin
        SelectedIndex := 0;
        Exit;
    end;
    Result := @Items[SelectedIndex];
end;

procedure TSideBarWidget.SetDelta(d: SizeInt);
begin
    Delta := d;
end;

procedure TSideBarWidget.SetItems(MenuItems: TGopherMenuItems);
begin
    Items := MenuItems;
end;

procedure TSideBarWidget.Reset;
begin
    SetLength(Items, 0);
    SelectedIndex := 0;
end;

procedure TSideBarWidget.SelectFirst(var Browser: TScroller);
var
    BrowserEnd: SizeInt = 0;
    BrowserObj: PBrowserWidget;
    BrowserRect: TRect;
    BrowserStart: SizeInt = 0;
begin
    BrowserObj := @Browser;
    BrowserRect := default(TRect);
    BrowserObj^.GetBounds(BrowserRect);
    BrowserStart := BrowserObj^.Delta.Y;
    BrowserEnd := BrowserStart + BrowserRect.B.Y;
    SelectedIndex := GetFirst;
    if Items[SelectedIndex].Position <= BrowserStart then
    begin
        BrowserObj^.ScrollTo(
            BrowserObj^.Delta.X,
            Items[SelectedIndex].Position
        );
    end;
end;

procedure TSideBarWidget.SelectLast(var Browser: TScroller);
var
    BrowserEnd: SizeInt = 0;
    BrowserObj: PBrowserWidget;
    BrowserRect: TRect;
    BrowserStart: SizeInt = 0;
begin
    BrowserObj := @Browser;
    BrowserRect := default(TRect);
    BrowserObj^.GetBounds(BrowserRect);
    BrowserStart := BrowserObj^.Delta.Y;
    BrowserEnd := BrowserStart + BrowserRect.B.Y;
    SelectedIndex := GetLast;
    if Items[SelectedIndex].Position >= BrowserEnd then
    begin
        BrowserObj^.ScrollTo(
            BrowserObj^.Delta.X,
            Items[SelectedIndex].Position - BrowserRect.B.Y + 1
        );
    end;
end;

procedure TSideBarWidget.SelectNext(var Browser: TScroller);
var
    BrowserEnd: SizeInt = 0;
    BrowserObj: PBrowserWidget;
    BrowserRect: TRect;
    BrowserStart: SizeInt = 0;
    I: SizeInt = 0;
    Last: SizeInt = 0;
begin
    BrowserObj := @Browser;
    BrowserRect := default(TRect);
    BrowserObj^.GetBounds(BrowserRect);
    BrowserStart := BrowserObj^.Delta.Y;
    BrowserEnd := BrowserStart + BrowserRect.B.Y;
    if Length(Items) = 0 then
    begin
        SelectedIndex := 0;
        Exit;
    end;
    Last := GetLast;
    for I := SelectedIndex + 1 to Length(Items) - 1 do
    begin
        if IsSelectable(Items[I]) = True then break;
    end;
    if I > Last then I := Last;
    SelectedIndex := I;
    if Items[SelectedIndex].Position >= BrowserEnd then
    begin
        BrowserObj^.ScrollTo(
            BrowserObj^.Delta.X,
            Items[SelectedIndex].Position - BrowserRect.B.Y + 1
        );
    end;
end;

procedure TSideBarWidget.SelectPrevious(var Browser: TScroller);
var
    BrowserEnd: SizeInt = 0;
    BrowserObj: PBrowserWidget;
    BrowserRect: TRect;
    BrowserStart: SizeInt = 0;
    First: SizeInt = 0;
    I: SizeInt = 0;
begin
    BrowserObj := @Browser;
    BrowserRect := default(TRect);
    BrowserObj^.GetBounds(BrowserRect);
    BrowserStart := BrowserObj^.Delta.Y;
    BrowserEnd := BrowserStart + BrowserRect.B.Y;
    if Length(Items) = 0 then
    begin
        SelectedIndex := 0;
        Exit;
    end;
    First := GetFirst;
    I := SelectedIndex - 1;
    while (I > 0) do
    begin
        if IsSelectable(Items[I]) = True then break;
        I := I - 1;
    end;
    if I < First then I := First;
    SelectedIndex := I;
    if Items[SelectedIndex].Position <= BrowserStart then
    begin
        BrowserObj^.ScrollTo(
            BrowserObj^.Delta.X,
            Items[SelectedIndex].Position
        );
    end;
end;


end.
